﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAccordionBus.CollectionGenericObjects;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    private readonly List<T> _collection;

    private int _maxCount;
    public int MaxCount {
        get => _maxCount;
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    public int Count => _collection.Count;


    public CollectionType GetCollectionType => CollectionType.List;

    public ListGenericObjects()
    {
        _collection = new();
    }

    public T Get(int position)
    {
        if (position < 0 || position >= _collection.Count || _collection == null || _collection.Count == 0) return null;

        return _collection[position];
    }

    public int Insert(T obj)
    {
        if (Count == _maxCount)
        {
            return -1;
        }

        _collection.Add(obj);
        return Count;
    }

    public int Insert(T obj, int position)
    {
        if (Count == _maxCount || position < 0 || position > Count)
        {
            return -1;
        }

        _collection.Insert(position, obj);
        return 1;
    }

    public T? Remove(int position)
    {
        if (_collection == null || position < 0 || position >= _collection.Count) return null;

        T? obj = _collection[position];
        _collection[position] = null;
        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; ++i)
        {
            yield return _collection[i];
        }
    }
}
