﻿using ProjectAccordionBus.CollectionGenericObjects;
using ProjectAccordionBus.Drawnings;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAccordionBus;

/// <summary>
/// 
/// </summary>
public partial class FormBusCollection : Form
{
    /// <summary>
    /// Хранилише коллекций
    /// </summary>
    private readonly StorageCollection<DrawningBus> _storageCollection;

    /// <summary>
    /// 
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBusCollection()
    {
        InitializeComponent();
        _storageCollection = new();
    }
    /// <summary>
    /// Добавление улучшенного троллейбуса
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddAccordionBus_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningAccordionBus));

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создания объекта</param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }

        Random random = new();
        DrawningBus drawningBus;
        switch (type)
        {
            case nameof(DrawningBus):
                drawningBus = new DrawningBus(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningAccordionBus):
                drawningBus = new DrawningAccordionBus(random.Next(100, 300), random.Next(1000, 3000), GetColor(random), GetColor(random),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        if (_company + drawningBus != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }

        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
    /// Выбор цвета
    /// </summary>
    /// <param name="random"></param>
    /// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }

        return color;
    }

    private void buttonAddBus_Click(object sender, EventArgs e)
    {
        FormBusConfig form = new();
        form.Show();
        form.AddEvent(SetBus);
    }

    private void SetBus(DrawningBus bus)
    {
        if (_company == null || bus == null) return;

        bus.SetPictureSize(pictureBox.Width, pictureBox.Height);

        if (_company + bus != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Объект не удалось добавить");
        }
    }

    private void buttonDeleteBus_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удалён");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null) return;

        DrawningBus? bus = null;
        int counter = 100;
        while (bus == null || counter > 0)
        {
            bus = _company.GetRandomObject();
            counter--;
        }

        if (bus == null) return;

        FormAccordionBus form = new()
        {
            SetBus = bus
        };
        form.ShowDialog();
    }

    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    private void comboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    private void maskedTextBox_MaskInputRejected(object sender, MaskInputRejectedEventArgs e)
    {

    }

    private void labelCollectionName_Click(object sender, EventArgs e)
    {

    }

    private void buttonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }
        _storageCollection.AddCollection(textBoxCollectionName.Text,
        collectionType);
        RefreshListBoxItems();
    }

    private void RefreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }


    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedItem == null) return;

        if (MessageBox.Show("Вы действительно хотите удалить выбранный элемент?",
            "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
        {
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RefreshListBoxItems();
            MessageBox.Show("Компания удалена");
        }
        else
        {
            MessageBox.Show("Не удалось удалить компанию");
        }
    }

    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0)
        {
            MessageBox.Show("Компания не выбрана");
            return;
        }

        ICollectionGenericObjects<DrawningBus?> collection = _storageCollection[listBoxCollection.SelectedItem?.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Компания не инициализирована");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new BusSharingService(pictureBox.Width, pictureBox.Height, collection);
                break;
        }

        panelCompanyTools.Enabled = true;
        RefreshListBoxItems();
    }

    private void saveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_storageCollection.SaveData(saveFileDialog.FileName))
            {
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

    private void loadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_storageCollection.LoadData(openFileDialog.FileName))
            {
                MessageBox.Show("Загрузка прошла успешно", "Реузльтат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RefreshListBoxItems();
            }
            else
            {
                MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
